/**
 * Copyright 2008,2009 Ivan SZKIBA
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * under the License.
 */

package net.sourceforge.tink.model.helpers;

import net.sourceforge.tink.model.TinkException;

import java.io.File;

public abstract class AbstractBuilder
{
    protected void requireDirectory(File dir, String attribute, String name) throws TinkException
    {
        requireValue(dir, attribute);
        if (!dir.isDirectory())
        {
            throw new TinkException(name + " directory doesn't exists: " + dir.getAbsolutePath());
        }
    }

    protected void requireFile(File file, String attribute, String name) throws TinkException
    {
        requireValue(file, name);
        if (!file.isFile())
        {
            throw new TinkException(name + " file doesn't exists: " + file.getAbsolutePath());
        }
    }

    protected void requireValue(Object value, String name) throws TinkException
    {
        if (value == null)
        {
            throw new TinkException(name + " must be set!");
        }
    }
}
