/**
 * Copyright 2008,2009 Ivan SZKIBA
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * under the License.
 */

package net.sourceforge.tink.model.impl;

import freemarker.cache.FileTemplateLoader;

import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;
import freemarker.template.Template;

import net.sourceforge.tink.model.FileObject;
import net.sourceforge.tink.model.TemplateData;
import net.sourceforge.tink.model.TinkContext;
import net.sourceforge.tink.model.TinkException;

import java.io.IOException;

public class TinkTransformer extends AbstractTinkComponent
{
    private static final String LOAD_ERROR = "Can' load template file: ";
    private static final String EXPAND_ERROR = "Can't expand tepmlate ";
    private Template template;
    private FileObject templateFile;

    public FileObject getTemplateFile()
    {
        return templateFile;
    }

    public void setTemplateFile(FileObject value)
    {
        templateFile = value;
    }

    @Override public void init(TinkContext context) throws TinkException
    {
        super.init(context);
        template = loadTemplate(getTemplateFile());
    }

    public void transform(TemplateData data, FileObject output) throws TinkException
    {
        transform(getTemplate(), data, output);
    }

    public void transform(FileObject templateFile, TemplateData data, FileObject output) throws TinkException
    {
        transform(loadTemplate(templateFile), data, output);
    }

    protected Template getTemplate() throws TinkException
    {
        return template;
    }

    protected Template loadTemplate(FileObject templateFile) throws TinkException
    {
        Configuration cfg = new Configuration();

        cfg.setDefaultEncoding(FileObject.CHARSET.name());
        cfg.setObjectWrapper(new DefaultObjectWrapper());
        try
        {
            cfg.setTemplateLoader(new FileTemplateLoader(templateFile.getParentFile()));

            return cfg.getTemplate(templateFile.getName());
        }
        catch (IOException x)
        {
            throw new TinkException(LOAD_ERROR + templateFile.getAbsolutePath(), x);
        }
    }

    protected void transform(Template template, TemplateData data, FileObject output) throws TinkException
    {
        try
        {
            template.process(data, output.openWriter());
        }
        catch (Exception x)
        {
            throw new TinkException(EXPAND_ERROR + template.getName() + " to file " + output.getAbsolutePath(), x);
        }
    }
}
